#!/usr/bin/env perl

#
# $Id$
#

# Copyright INRA

# Mathieu.Gautier@supagro.inra.fr
# Astrid.Cruaud@supagro.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#

=head1 NAME

RADIS.pl

=head1 SYNOPSIS

$RADIS/bin/RADIS.pl [--cfg filename] --datadir rooted_dirname --outdir rooted_dirname [--ncpus integer]

=head1 DESCRIPTION

=cut

BEGIN
{
	$ENV{LANG}="C";
	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../lib");
	$ENV{RADIS} = "$dirprg/.." if ( ! defined ($ENV{RADIS}) );
	our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

use Getopt::Long;
use ParamParser;
use Data::Dumper;
use File::Path;
use File::Copy;
use General;
use GeneralBioinfo;
require "radis-lib.pl";

use strict;
use warnings;

my $DEFAULT_NCPUS = 2;
my $LOGGER_LEVEL_EXEC = 'DE';
my $DEBUG=0;

MAIN:
{
	print "$0 ".join(' ',@ARGV)."\n";
	my $o_param =  New ParamParser('GETOPTLONG',\&Usage,"help","cfg=s","datadir=s","outdir=s","ncpus=i","radis_process_radtags_param=s","radis_maxtaglen=i",'radis_enzsitelen=i','steps=s','listoffastqfiles=s','radis_npbloci_cutoff=i','radis_bwa_seedlen=i','debug','interleave_list_of_files','without-raxml');
	$o_param->SetBehaviour('use_substitution_table');
	$o_param->SetUnlessDefined('cfg',"$ENV{RADIS}/cfg/RADIS.cfg");
	$o_param->AssertFileExists('cfg','outdir');
	$o_param->AssertDirExists('outdir');
	$o_param->AssertFullPath('outdir','cfg');
	&GetExePath("R");
	# il faut verifier la dependence R::ape
	$o_param->SetSubstitution('%i',\$ENV{RADIS});
	# pourquoi pas $o_param->SetSubstitution('%i',$ENV{RADIS});
	$o_param->SetUnlessDefined('ncpus',$DEFAULT_NCPUS);
	$o_param->SetUnlessDefined('steps',"12");
	$o_param->SetSubstitution('%n',$o_param->Get('ncpus'));
	$o_param->Update($o_param->Get('cfg'),'U');

	$main::DEBUG = 1 if ( $o_param->IsDefined('debug') );

	my $outdir  = $o_param->Get('outdir');
	my $steps=$o_param->Get('steps');

	#foreach my $exe_key ('radis_prg_step1','radis_prg_step2','stacks_process_radtags','stacks_clone_filter','stacks_ustacks','stacks_cstacks','raxml_prg','bwa_index','bwa_mem','samtools')
	foreach my $exe_key ('radis_prg_step1','radis_prg_step2','stacks_process_radtags','stacks_clone_filter','stacks_ustacks','stacks_cstacks','raxml_prg')
	{
		my $exe = $o_param->Get($exe_key);
		next if ( $exe_key eq 'raxml_prg' && $o_param->IsDefined('without-raxml') );
		$exe=~s/\s.*//;
		if ( &GetExePath($exe) eq '' )
		{
			Carp::croak("Error: $exe_key program not found or not executable\n");
		}
	}

	# AVANT de continuer il faudrait verifier que tous les binaires sont bien dispo et executables

	my $cde = "";
	my $ret = 0;;

	if ( $steps =~ /1/ )
	{
		$cde = $o_param->Get('radis_prg_step1');
		$cde .= &GetCommonParams($o_param);
		foreach my $param_step1 ('radis_process_radtags_param','radis_maxtaglen','radis_enzsitelen','datadir','debug','interleave_list_of_files')
		{
			$cde .= " --$param_step1 ".$o_param->Get($param_step1)." " if ( $o_param->IsDefined($param_step1) );
		}
		&PrintBanner("RADIS step1: data cleaning");
		$ret = &System($cde,$main::DEBUG,*STDERR);
		$o_param->Set('listoffastqfiles',"\'$outdir/*/04_PUR/*fq_1\'");
	}
	if ( $steps =~ /2/ && ! $ret )
	{
		if ( -e "$outdir/radis_step1.cfg") # pour recuperer radis_bwa_seedlen
		{
			$o_param->Update("$outdir/radis_step1.cfg",'U');
		}
		my $loutdir =  $outdir."/ASSEMBLY_STACKS";
		mkdir $loutdir;
		$o_param->Set('outdir',$loutdir);
		$cde = $o_param->Get('radis_prg_step2');
		$cde .= &GetCommonParams($o_param);
		foreach my $param_step1 ('listoffastqfiles','radis_bwa_seedlen','radis_npbloci_cutoff')
		{
			$cde .= " --$param_step1 ".$o_param->Get($param_step1)." " if ( $o_param->IsDefined($param_step1) );
		}
		foreach my $param_step1 ('without-raxml','debug')
		{
			$cde .= " --$param_step1 " if ( $o_param->IsDefined($param_step1) );
		}
		&PrintBanner("RADIS step2: data analysis");
		$ret = &System($cde,$main::DEBUG,*STDERR);
		$o_param->Set('outdir',$outdir);
	}
	if ( ! $ret )
	{
		&PrintBanner("Finished");
	}
	else
	{
		&PrintBanner("Stopped with an error");
	}
}

sub PrintBanner
{
my($txt)=@_;
	
	my @a_date = `date`;
	chomp(@a_date);
	print "===============================================================================\n";
	print "      $txt\t$a_date[0]\n";
	print "===============================================================================\n";
}

sub GetCommonParams
{
my($o_param)=@_;

	my $cde = '';
	foreach my $param ('cfg','outdir','ncpus')
	{
		$cde .= " --$param ".$o_param->Get($param)." " if ( $o_param->IsDefined($param) );
	}
	return($cde);
}

=head2 procedure Usage
 
 Title        : Usage
 Usage        : &Usage()
 Prerequisite : none
 Fonction     : print the program usage
 Returns      : none
 Args         :	none
 Globals      : none

=cut

sub Usage
{
my $ncpus = $DEFAULT_NCPUS;

print STDERR<<END
$0

[Mandatory]
	--help
	--outdir     dirname        full/rooted path
[Optional]
	--cfg        filename       [$ENV{RADIS}/cfg/RADIS.cfg] configuration file 
	--ncpus      integer        [$ncpus] number of cpus
	--interleave_list_of_files  use this flag when the ls command groups all read1 files and then all read2 files 
	                            (default behaviour is file1_read1, file1_read2, file2_read1, file2_read2, etc)

In addition

Step1: data cleaning
[Mandatory]
	--datadir    dirname  		location of data files:
	                                  *fastq.gz *.fq *.fastq *.fastq
									  barcodes_lib_names.txt (two formats allowed: 1) tag_read1<tab>sample_name 2) tag_read1<tab>tag_read2<tab>sample_name)
Step2: data analyses

[Optional]
	--listoffastqfiles  string        the value of the string can be: a file of files (.fof extension), a single file or a quoted string '../*/*PUR/*fq_1'
	--radis_npbloci_cutoff integer    [cf $ENV{RADIS}/cfg/RADIS.cfg] cutoff to define a stack as 'containing likely paralogs', set radis_npbloci_cutoff to 0 to skip these analyses
	--without-raxml                   skip RAxML analyses
END
;
#[Optional]
#	--radis_maxtaglen  integer        [cf $ENV{RADIS}/cfg/RADIS.cfg] maximum length of the tag (post demultiplexing)
}
