#!/usr/bin/env perl

#
# $Id$
#

# Copyright INRA

# Mathieu.Gautier@supagro.inra.fr
# Astrid.Cruaud@supagro.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#

=head1 NAME

RADIS_step1_data_cleaning.pl

=head1 SYNOPSIS

$RADIS/bin/RADIS_step1_data_cleaning.pl [--cfg filename] --datadir rooted_dirname --outdir rooted_dirname [--ncpus integer]

=head1 DESCRIPTION

=cut

BEGIN
{
	$ENV{LANG}="C";
	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../lib");
	$ENV{RADIS} = "$dirprg/.." if ( ! defined ($ENV{RADIS}) );
	our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

use Getopt::Long;
use ParamParser;
use Data::Dumper;
use File::Path;
use File::Copy;
use General;
use GeneralBioinfo;
require "radis-lib.pl";
use Carp;

use strict;
use warnings;

my $DEFAULT_NCPUS = 2;
my $LOGGER_LEVEL_EXEC = 'DE';
my $DEBUG = 0;

my $READ2 = &TRUE; # on peut pas le desactiver dans stacks

MAIN:
{
	my $o_param =  New ParamParser('GETOPTLONG',\&Usage,"help","cfg=s","datadir=s","outdir=s","ncpus=i","stacks_process_radtags_param=s","radis_readlen=i",'radis_enzsitelen=i','debug','interleave_list_of_files');
	$o_param->SetBehaviour('use_substitution_table');
	$o_param->SetUnlessDefined('cfg',"$ENV{RADIS}/cfg/RADIS.cfg");
	$o_param->AssertFileExists('cfg','datadir','outdir');
	$o_param->AssertDirExists('outdir','datadir');
	$o_param->AssertFullPath('datadir','outdir','cfg');
	$o_param->SetSubstitution('%i',\$ENV{RADIS});
	$o_param->SetUnlessDefined('ncpus',$DEFAULT_NCPUS);
	$o_param->SetSubstitution('%n',$o_param->Get('ncpus'));
	$o_param->Update($o_param->Get('cfg'),'U');
	#my $radis_enzsitelen = $o_param->Get('radis_enzsitelen');
	#&NoSpace(\$radis_enzsitelen);
	#&AssertInteger($radis_enzsitelen);

	$main::DEBUG = 1 if ( $o_param->IsDefined('debug') );

	my $outdir  = $o_param->Get('outdir');

	my @a_ls = `ls $outdir/* 2>/dev/null`;
	if ( scalar(@a_ls) )
	{
		Carp::croak("Error: outdir must be an empty dir\n");
	}
	my $datadir = $o_param->Get('datadir');
	my $ncpus=$o_param->Get('ncpus');
	chdir($outdir);

	#$o_param->Print();

	my $barcodelibnames = $o_param->Get('radis_default_barcode_lib_names');
	&NoSpace(\$barcodelibnames);
	$barcodelibnames = "$datadir/$barcodelibnames";

	if ( ! -e $barcodelibnames )
	{
		Carp::croak("No ".$o_param->Get('radis_default_barcode_lib_names')." found in $datadir");
	}
	$o_param->AssertFileExists('stacks_process_radtags','stacks_clone_filter');
	$o_param->AssertDefined('stacks_process_radtags_param');

	my $radis_readlen = $o_param->Get('radis_readlen');
	&NoSpace(\$radis_readlen);
	&AssertInteger($radis_readlen);

	my @a_fqfiles = &GetListOfFiles($datadir,"*.fastq *.fq *.fastq.gz *.fq.gz");
	if ( $o_param->IsDefined('interleave_list_of_files') )
	{
		my @a_fqfiles_raw = @a_fqfiles;
		@a_fqfiles=();
		my $offset = scalar(@a_fqfiles_raw)/2;
		for(my $i=0;$i<$offset;$i++)
		{
			push(@a_fqfiles,$a_fqfiles_raw[$i]);
			push(@a_fqfiles,$a_fqfiles_raw[$i+$offset]);
		}
	}
	my $lenfn=length($a_fqfiles[0])+10;

	my %h_barcodefiles=();
	# pour chaque longueur de tag on cree un fichier avec les tags
	my $maxbarcodelen_read1=0;
	my $maxbarcodelen_read2=0;
	my $mode_process_tags = &DumpTagsFiles($barcodelibnames,\%h_barcodefiles,$outdir,\$maxbarcodelen_read1,\$maxbarcodelen_read2);
	#print STDERR "$maxbarcodelen_read1 // $maxbarcodelen_read2\n";
	my $radis_nttrim_read1_5p = $o_param->Get('radis_nttrim_read1_5p');
	&NoSpace(\$radis_nttrim_read1_5p);
	&AssertInteger($radis_nttrim_read1_5p);
	my $radis_nttrim_read1_3p = $o_param->Get('radis_nttrim_read1_3p');
	&NoSpace(\$radis_nttrim_read1_3p);
	&AssertInteger($radis_nttrim_read1_3p);
	my $radis_nttrim_read2_3p = $o_param->Get('radis_nttrim_read2_3p');
	&NoSpace(\$radis_nttrim_read2_3p);
	&AssertInteger($radis_nttrim_read2_3p);
	my $radis_nttrim_read2_5p = $o_param->Get('radis_nttrim_read2_5p');
	&NoSpace(\$radis_nttrim_read2_5p);
	&AssertInteger($radis_nttrim_read2_5p);

	&PrintJobStatus("Step 0.0: Checking parameters","done",&TRUE);
	&PrintJobStatus("Step 0.0: Dumping pairs of files","started",&TRUE);
	printf STDERR "%-${lenfn}s%${lenfn}s\n",'Read1','Read2';
	for(my $i=0;$i<scalar(@a_fqfiles);$i+=2)
	{
		printf STDERR "%-${lenfn}s%${lenfn}s\n",$a_fqfiles[$i],$a_fqfiles[$i+1];
	}
	print STDERR "Warning: consider adding the --interleave_list_of_files flag in case of incorrect pairing\n";
	&PrintJobStatus("Step 0.0: Dumping pairs of files","done",&TRUE);

	#====== Filtre qualite ==============

	my $msg = sprintf("Step 1.0: Read filtering (%d files)",scalar(@a_fqfiles));
	&PrintJobStatus($msg,"started",&TRUE);
	my %sh_statsgoodreads:shared=();
	&Multithreading(\&SliceFastqIlluminaFilter,$ncpus,\@a_fqfiles,\%sh_statsgoodreads,$outdir);
#$VAR1 = 'Good_R2.fq.gz';
#$VAR2 = '1000000        1000000';
#$VAR3 = 'Good_R1.fq.gz';
#$VAR4 = '1000000        1000000';

	my $tot_goodR1 = 0;
	my $tot_goodR2 = 0;
	my $tot_R1 = 0;
	my $tot_R2 = 0;
	for(my $i=0;$i<scalar(@a_fqfiles);$i+=2)
	{
		my ($f1) = $a_fqfiles[$i]  =~ /.+\/(.+)/;
		my($good,$tot)=split("\t",$sh_statsgoodreads{$f1});
		$tot_R1 += $tot;
		$tot_goodR1 += $good;
		my ($f2) = $a_fqfiles[$i+1]  =~ /.+\/(.+)/;
		($good,$tot)=split("\t",$sh_statsgoodreads{$f2});
		$tot_R2 += $tot;
		$tot_goodR2 += $good;
	}
	$sh_statsgoodreads{"_Total_read1"} = "$tot_goodR1\t$tot_R1";
	$sh_statsgoodreads{"_Total_read2"} = "$tot_goodR2\t$tot_R2";
	&DumpHash("$outdir/1.0.statsgoodreads.tsv",\%sh_statsgoodreads,"Read file","# good reads\t# reads");
	&PrintJobStatus("Step 1.0: $outdir/1.0.statsgoodreads.tsv ","done",&TRUE);

	#====== Demultiplexage ==============

	$msg = sprintf("Step 1.1: Stacks/process_radtags");
	&PrintJobStatus($msg,"started",&TRUE);
	@a_fqfiles = &GetListOfFiles($outdir,"Good*gz");
	my @a_cde = ();
	if ( scalar(@a_fqfiles) > 2 ) # It seems that I must merge files before process_radtags
	{
		my $lst1 = '';
		my $lst2 = '';
		my $out1 = "$outdir/Good_R1.fq.gz";
		my $out2 = "$outdir/Good_R2.fq.gz";
		for(my $i=0;$i<scalar(@a_fqfiles);$i+=2)
		{
			$lst1 .= " $a_fqfiles[$i] ";
			$lst2 .= " $a_fqfiles[$i+1] ";
		}
		push(@a_cde," gzip -cd $lst1 | gzip -c > $out1 ");
		push(@a_cde," gzip -cd $lst2 | gzip -c > $out2 ");
		&Multithreading(\&SliceSystem,$ncpus,\@a_cde,undef);
		&System("rm $outdir/Good_?????_*gz");
		@a_fqfiles = &GetListOfFiles($outdir,"Good*gz");
	}
	my $param = $o_param->Get('stacks_process_radtags_param');
	if ( $mode_process_tags > 1 )
	{
		$param .= " --inline_inline " if( $param !~ /inline_inline/ );
	}
	print "REM: stacks_process_radtags_param=$param \n";
	#(cf --stacks_process_radtags_param or --cfg to modify parameters)
	@a_cde = ();
	my @a_barcodelengths=keys(%h_barcodefiles);
	for(my $i=0;$i<scalar(@a_fqfiles);$i+=2) # ==> cf petite contraintes de nomage
	{
		for(my $f=0;$f<scalar(@a_barcodelengths);$f++)
		{
			my $loutdir = "$outdir/$a_barcodelengths[$f]";
			mkdir $loutdir;
			push(@a_cde,$o_param->Get('stacks_process_radtags')." -1 $a_fqfiles[$i] -2 $a_fqfiles[$i+1] -o $loutdir -b $h_barcodefiles{$a_barcodelengths[$f]} $param");
		}
	}
	my $RUN = 1;
	if ( $RUN )
	{
	&Multithreading(\&SliceSystem,$ncpus,\@a_cde,undef);
	my %h_statprocesstags = ();
	@a_cde =();
	for(my $f=0;$f<scalar(@a_barcodelengths);$f++)
	{
		my $loutdir = "$outdir/$a_barcodelengths[$f]";
		mkdir "$loutdir/01_REM";
		mkdir "$loutdir/01_DMP"; # demultiplexe ok
		&System("cd $loutdir ; mv *rem*fq *rem*fq.gz 01_REM",$main::DEBUG,undef);
		my @a_wc = `wc -l $loutdir/*fq 2> /dev/null`;
		if ( ! scalar(@a_wc) )
		{
			my @a_ls = `ls $loutdir/*fq.gz`;
			chomp(@a_ls);
			foreach my $f (@a_ls)
			{
				push(@a_wc,`gzip -cd $f | wc -l`." $f");
			}
		}
		chomp(@a_wc);
		foreach my $l (@a_wc)
		{
			next if ( $l =~ /total/ );
			my ($count,$file)=split(' ',$l);
			my($shortf)=$file=~/.+\/(.+)/;
			#print STDERR "$count,$file,$l,$shortf\n";
			$h_statprocesstags{"$a_barcodelengths[$f]\t$file"}=$count/4;
		}
		# en principe ca gere les fq et les fq.gz
		push(@a_cde,"cd $loutdir ; gzip -f 01_REM/*fq");
		push(@a_cde,"cd $loutdir ; mv *fq *fq.gz  01_DMP ; gzip -f 01_DMP/*fq");
	}
	&Multithreading(\&SliceSystem,$ncpus,\@a_cde,undef);
#$VAR1 = '5      /data/RADIS/TEST3/5/sample_GCCGC.2.fq';
#$VAR2 = '31734';
	&DumpHash("$outdir/1.1.statsprocesstags.tsv",\%h_statprocesstags,"Tag length\tread file","# reads");
	}
	&PrintJobStatus("Step 1.1: $outdir/1.1.statsprocesstags.tsv ","done",&TRUE);

	sleep(10);

	#===== Trim ===============

	$msg = sprintf("Step 1.2: triming ",scalar(keys(%h_barcodefiles)));
	&PrintJobStatus($msg,"starting",&TRUE);


	my $maxtaglen_read1  = $radis_readlen-$radis_nttrim_read1_5p-$radis_nttrim_read1_3p-$maxbarcodelen_read1;
	my $maxtaglen_read2  = $radis_readlen-$radis_nttrim_read2_5p-$radis_nttrim_read2_3p-$maxbarcodelen_read2;

	#my $f_cfg = &GetStreamOut("$outdir/radis_step1.cfg");
	#print $f_cfg "radis_bwa_seedlen=$maxtaglen\n";
	#$f_cfg->close;

	print "REM: maxbarcodelen_read1=$maxbarcodelen_read1 => length of the read1 tag=radis_readlen($radis_readlen)-radis_nttrim_read1_3p($radis_nttrim_read1_3p)-radis_nttrim_read1_5p($radis_nttrim_read1_5p)-maxbarcodelen_read1($maxbarcodelen_read1)=$maxtaglen_read1\n";
	for(my $f=0;$f<scalar(@a_barcodelengths);$f++)
	{
		my $taglen=$a_barcodelengths[$f];
		my $loutdir = "$outdir/$taglen";
		my $outtrim = "$loutdir/02_TRIM";
		mkdir $outtrim; 
		mkdir "$loutdir/03_FIL"; 
		mkdir "$loutdir/04_PUR"; 
		@a_fqfiles = &GetListOfFiles("$loutdir/01_DMP","*.1.fq.gz");
		if ( $RUN )
		{
			#&Multithreading(\&SliceTrim,$ncpus,\@a_fqfiles,undef,$enzsitelen+1,$maxtaglen,$outtrim);
			&Multithreading(\&SliceTrim,$ncpus,\@a_fqfiles,undef,$radis_nttrim_read1_5p+1,$maxtaglen_read1,$outtrim);
		}
		if ($READ2)
		{
			@a_fqfiles = &GetListOfFiles("$loutdir/01_DMP","*.2.fq.gz");
			if ( $RUN )
			{
				&Multithreading(\&SliceTrim,$ncpus,\@a_fqfiles,undef,$radis_nttrim_read2_5p+1,$maxtaglen_read2,$outtrim);
			}
		}
	}
	&PrintJobStatus("Step 1.2: ls $outdir/*/02_TRIM/*.?.trim.fq.gz ","done",&TRUE);


	#===== Clone Filter ===============

	&PrintJobStatus("Step 1.3: Stacks/clone_filter ","starting",&TRUE);
	@a_fqfiles = &GetListOfFiles("$outdir/*/02_TRIM","*.?.trim.fq.gz");
	@a_cde = ();
	my $step = ($READ2) ? 2 : 1;
	for(my $i=0;$i<scalar(@a_fqfiles);$i+=$step)
	{
		my($dir,$file)=$a_fqfiles[$i]=~/(.+)\/(.+)/; 
		if ( $READ2 )
		{
			push(@a_cde,$o_param->Get('stacks_clone_filter')." -1 $a_fqfiles[$i] -2 $a_fqfiles[$i+1] -o $dir/../03_FIL -i gzfastq > $dir/../03_FIL/$file.clone_filter.log");
		}
		else
		{
			push(@a_cde,$o_param->Get('stacks_clone_filter')." -1 $a_fqfiles[$i]                     -o $dir/../03_FIL -i gzfastq > $dir/../03_FIL/$file.clone_filter.log");
		}
	}
	if ( $RUN )
	{
		&Multithreading(\&SliceSystem,$ncpus,\@a_cde,undef);
	}
	my %h_countclonefilter=();
	my @a_clone_filter_logs = &GetListOfFiles("$outdir/*/03_FIL","*.clone_filter.log");
	my $total=0;
	foreach my $logfile (@a_clone_filter_logs)
	{
		my $f_in = &GetStreamIn($logfile);
		while(my $l=<$f_in>)
		{
			chomp($l);
			next if ( $l !~ /\d+\t\d+/ );
			my ($cl,$count) = split(/\t/,$l);
			$h_countclonefilter{$cl}+=$count;
			$total+=$count;
		}
		$f_in->close;
	}
	#foreach my $cl (keys(%h_countclonefilter))
	#{
	#	$h_countclonefilter{$cl} .= "\t".sprintf("%.4f",$h_countclonefilter{$cl}*100/$total);
	#}
	&DumpHash("$outdir/1.3.countclonefilter.tsv",\%h_countclonefilter,"NumClones","Count");
	&PrintJobStatus("Step 1.3: $outdir/1.3.countclonefilter.tsv ","done",&TRUE);

	#===== Rename ===============

	&PrintJobStatus("Step 1.4: rename ","starting",&TRUE);
	@a_cde = ();
	my $f_in=&GetStreamIn($barcodelibnames);
	if ( $mode_process_tags == 1 )
	{
		while(my $l=<$f_in>)
		{
			chomp($l);
			my ($site,$name)=split(/\t/,$l);
			my $len = length($site);
			&System("mv $outdir/$len/03_FIL/sample_$site.*fq_1 $outdir/$len/04_PUR/$name.fq_1",$main::DEBUG,undef);
			if ( $READ2 )
			{
				&System("mv $outdir/$len/03_FIL/sample_$site.*fq_2 $outdir/$len/04_PUR/$name.fq_2 ; gzip  $outdir/$len/04_PUR/$name.fq_2",$main::DEBUG,undef); # je les compresse on ne les utilise pas
			}
		}
	}
	else
	{
		while(my $l=<$f_in>)
		{
			chomp($l);
			my ($site1,$site2,$name)=split(/\t/,$l);
			&System("mv $outdir/PBC/03_FIL/$name*fq_1 $outdir/PBC/04_PUR/$name.fq_1",$main::DEBUG,undef);
			if ( $READ2 )
			{
				&System("mv $outdir/PBC/03_FIL/$name*fq_2 $outdir/PBC/04_PUR/$name.fq_2 ; gzip  $outdir/PBC/04_PUR/$name.fq_2",$main::DEBUG,undef); # je les compresse on ne les utilise pas
			}
		}
	}
	$f_in->close;
	&PrintJobStatus("Step 1.4: count usable reads ","starting",&TRUE);
	@a_cde = ();
	my @a_fq_1 = `ls $outdir/*/04_PUR/*.fq_1`;
	chomp(@a_fq_1);
	my %sh_countreads:shared=();
	&Multithreading(\&SliceCountReads,$ncpus,\@a_fq_1,\%sh_countreads);
	my %h_barc = ();
	if ($mode_process_tags==1)
	{
		&TabToHash($barcodelibnames,\%h_barc,2,1);
	}
	else
	{
		&TabToHash($barcodelibnames,\%h_barc,3,1);
	}
	my $f_purlog = &GetStreamOut("$outdir/1.4.statspurereads.tsv");
	print $f_purlog "barcode\tsample_code\tcount pure reads\tfile\n";
	foreach my $file (sort keys(%sh_countreads))
	{
		my($dir,$sample)=$file=~/(.+)\/(.+).fq_1/;
		print $f_purlog "$h_barc{$sample}\t$sample\t$sh_countreads{$file}\t$file\n";
	}
	$f_purlog->close;
	&PrintJobStatus("Step 1.4: ls $outdir/1.4.statspurereads.tsv ","done",&TRUE);
	&PrintJobStatus("Step 1.4: ls $outdir/*/04_PUR/*.fq_1 ","done",&TRUE);
	exit(0);
}

sub SliceCountReads
{
my($i,$ra_tasks,$ra_inputs,$rsh_countreads)=@_;

	my ($nb_reads,@a_tmp) = split(' ',`wc $$ra_tasks[$i]`);
	$$rsh_countreads{$$ra_tasks[$i]}=$nb_reads/4;
}

sub DumpHash
{
my($outfile,$rh_data,$key_labels,$val_labels)=@_;

	my $f_out = &GetStreamOut($outfile);
	print $f_out "$key_labels\t$val_labels\n";
	foreach my $key (sort { if ( $a=~/^\d+$/ && $b=~/^\d+$/ ) { return $a <=> $b } else { return $a cmp $b } } keys(%$rh_data))
	{
		print $f_out "$key\t$$rh_data{$key}\n";
	}
	$f_out->close;
}


# verif identique trim_fastq.awk faite
sub SliceTrim
{
my($i,$ra_tasks,$ra_inputs,$rsh_,$start,$len,$outtrim)=@_;

	my $in = $$ra_tasks[$i];
	my ($dir,$file)=$in=~/(.+)\/(.+)/;
	my $out = "$outtrim/$file";
	$out=~s/.fq.gz$/.trim.fq.gz/;
	my $f_in  = &GetStreamIn($in);
	my $f_out  = &GetStreamOut($out);
	my $cpt=0;
	while(my $l=<$f_in>)
	{
		chomp($l);
		if ( $cpt++ % 2 == 1 )
		{
			my $ll = substr($l,$start-1,$len);
			if ( ! defined($ll) )
			{
				print STDERR "$out\n$cpt\n$l\n$start-1,$len\n";
				exit(1);
			}
			$l = $ll;
		}
		print $f_out "$l\n";
	}
	$f_out->close;
	$f_in->close;
}


sub DumpTagsFiles
{
my($barcodelibnames,$rh_barcodefiles,$outdir,$r_maxbarcodelen_read1,$r_maxbarcodelen_read2)=@_;

	my $pat = "$outdir/barcodes%dbp.pools";
	my $f_in = &GetStreamIn($barcodelibnames);
	#AAACAA  JRAS03806
	#AAGGG   JRAS03887
	my %h_io=();
	my $mode = 2;
	while(my $l=<$f_in>)
	{
		chomp($l);
		my @a_l = split(/\t/,$l);
		$mode = scalar(@a_l)-1;
		if( $mode == 1 )
		{
			my ($tag,$sample)=split(/\t/,$l);
			my $len = length($tag);
			$$r_maxbarcodelen_read1 = $len if ( $len > $$r_maxbarcodelen_read1 );
			if ( ! defined($h_io{$len}) )
			{
				my $bc_len = sprintf($pat,$len);
				$h_io{$len} = &GetStreamOut($bc_len);
				$$rh_barcodefiles{$len}=$bc_len;
			}
			select($h_io{$len});
			print "$tag\n";
		}
		else
		{
			$$r_maxbarcodelen_read1 = length($a_l[0]) if ( length($a_l[0]) > $$r_maxbarcodelen_read1 );
			$$r_maxbarcodelen_read2 = length($a_l[1]) if ( length($a_l[1]) > $$r_maxbarcodelen_read2 );
		}
	}
	$f_in->close;
	if ( $mode == 2 )
	{
		&System("cp $barcodelibnames $outdir/barcodes.pairs");
		$$rh_barcodefiles{PBC}="$outdir/barcodes.pairs";
	}
	foreach my $len (keys(%h_io))
	{
		$h_io{$len}->close;
	}
	select(STDOUT);
	return($mode);
}

sub SliceFastqIlluminaFilter
{
my($i,$ra_tasks,$ra_inputs,$rsh_statsgoodreads,$outdir)=@_;

	my $file = $$ra_tasks[$i];
	my ($f) = $file =~ /.+\/(.+)/;
	my $good_reads = sprintf("$outdir/Good_%05d_$f",$i);
	$good_reads .= ".gz" if ( $f !~ /.gz$/ );
	my $f_good = &GetStreamOut($good_reads);
	my $f_fq = &GetStreamIn($file);
	my $cpt=0;
	my $ok;
	my $cpt_reads=0;
	my $cpt_good_reads=0;

	while(my $l=<$f_fq>)
	{
		if ( $cpt++ % 4 == 0 )
		{
			$ok = &FALSE;
			$cpt_reads++;
			if ( $l =~ / [^:]+:N:/ )
			{
				$ok = &TRUE;
				$cpt_good_reads++;
			}
		}
		print $f_good $l if ( $ok );
	}

	$f_good->close;
	$f_fq->close;
	$$rsh_statsgoodreads{$f} = "$cpt_good_reads\t$cpt_reads";
}

=head2 procedure Usage
 
 Title        : Usage
 Usage        : &Usage()
 Prerequisite : none
 Fonction     : print the program usage
 Returns      : none
 Args         :	none
 Globals      : none

=cut

sub Usage
{
print STDERR<<END
$0
[Mandatory]
	--help
	--datadir    dirname  		location of data files:  barcodes_lib_names.txt, *fastq.gz *.fq *.fastq *.fastq
	--outdir     dirname 		WARNING: it must be empty

[Optional]
	--cfg        filename  		[$ENV{RADIS}/cfg/RADIS.cfg] configuration file
	--ncpus      integer        [$DEFAULT_NCPUS] number of cpus
	--interleave_list_of_files  use this flag when the ls command groups all read1 files and then all read2 files 
                                (default behaviour is file1_read1, file1_read2, file2_read1, file2_read2, etc)
END
;
#	--radis_enzsitelen integer  [cf $ENV{RADIS}/cfg/RADIS.cfg] length of the enzyme
#--radis_maxtaglen  integer  [cf $ENV{RADIS}/cfg/RADIS.cfg] maximum length of the tag (post demultiplexing)
}
