#============================================================================================================
#============================================================================================================
#============================================================================================================

# pour le moment je colle le plus possible au script initial (qui faisait trois fois la lecture pour Mean/Min/Max) mais on pourrait generaliser ou utiliser Statistics::R

sub DistanceR
{
my($dir,$ext)=@_;

	my $rscript = "$dir/Distance.r";
	my $f_r = &GetStreamOut($rscript);
	select($f_r);

print<<END
require(ape)

DistanceMean<-function(filename){
    read.dna(filename,format="sequential") -> aln
    dist.dna(aln, model = "N", pairwise.deletion = TRUE) -> Mat_aln # p-distance matrix (pairwise.deletion default = FALSE : is to delete the sites with at least one missing data for all sequences)
    MeanDist <- mean(Mat_aln) #moyenne des parwise
    return(MeanDist)
} 
DistanceMin<-function(filename){
    read.dna(filename,format="sequential") -> aln
    dist.dna(aln, model = "N", pairwise.deletion = TRUE) -> Mat_aln
    MinDist <- min(Mat_aln)
    return(MinDist)
} 
DistanceMax<-function(filename){
    read.dna(filename,format="sequential") -> aln
    dist.dna(aln, model = "N", pairwise.deletion = TRUE) -> Mat_aln
    MaxDist <- max(Mat_aln)
    return(MaxDist)
} 
END
;

	foreach my $stat ('Mean','Min','Max')
	{
	#	print "======== Distance $stat ==============\n";

print<<END
liste_fichier<-list.files(pattern="$ext\$") #files of the current folder whose filename extension is $ext are listed 
length(liste_fichier) #check you have all the loci you want in the list
sapply(liste_fichier,Distance$stat) -> RESULTS # apply function to all files and save results in RESULTS

length(RESULTS) # check all loci have be taken into account
summary(RESULTS) # mean,median,25th and 75th quartiles,min,max


#export results
write(RESULTS,file="RESULTS.Distance.$stat.txt")

#histogramms
pdf("RESULTS.Distance.$stat.pdf", height=10,width=10)
h = hist(RESULTS)
h\$density = h\$counts/sum(h\$counts)*100
effectif=table(RESULTS)
abline(v = mean(RESULTS), col = "red", lwd = 2)
abline(v = median(RESULTS), col = "green", lwd = 2)
dev.off()

END
;
	}
	$f_r->close;
	my $cde = "cd $dir ; R --vanilla < $rscript > $rscript.stdout 2> $rscript.stderr";
	&System($cde,${main::DEBUG},undef);
	select(STDOUT);

}

#============================================================================================================
#============================================================================================================
#============================================================================================================

sub CatalogStatR
{
my($outdir,$batch_tsv)=@_;

	my $rscript = "$outdir/catstats.r";
	my $f_r = &GetStreamOut($rscript);
	select($f_r);

print<<END
mat_res=as.matrix(read.table("$batch_tsv",row.names=1,h=T))
sink("$batch_tsv.summary", append=FALSE, split=FALSE)
row<-data.frame(rowSums(mat_res>0))
col<-data.frame(colSums(mat_res>0))
names(col)<-c("counts")
cat("Number of samples in catalog before sample selection=",dim(mat_res)[2],"\\n")
cat("Number of loci in catalog before loci selection=",dim(mat_res)[1],"\\n")
cat("\\nSummary statistics for the number of sample per locus\\n")
row<-summary(row);attr(row,"dimnames")[[2]]<-"";row
cat("\\nNumber of loci per sample\\n")
col
cat("\\nSummary statistics for the number of loci per sample\\n")
col<-summary(col);attr(col,"dimnames")[[2]]<-"";col
cat("\\nPercentage of loci for which samples have more than one sequence\\n")
100*colSums(mat_res>1)/colSums(mat_res>=0)
cat("\\nPercentage of loci for which samples have three or more  sequences\\n")
100*colSums(mat_res>2)/colSums(mat_res>=0)
cat("\\n")
END
;
	foreach $cutoff (5,10,25,35,50,65,75,85)
	{
print<<END
cat("Number of loci in which at least $cutoff% of the samples are represented=",sum(rowSums(mat_res>0)>=dim(mat_res)[2]*$cutoff/100),"\\n")
END
;
	}
print<<END
cat("Number of loci in which all samples are represented=",sum(rowSums(mat_res>0)==dim(mat_res)[2]),"\\n")
END
;

	$f_r->close;
	my $cde = "cd $outdir ; R --vanilla < $rscript > $rscript.stdout 2> $rscript.stderr ";
	&System($cde,${main::DEBUG},undef);
	select(STDOUT);
}

#============================================================================================================
#============================================================================================================
#============================================================================================================

# je genere le script R, comme ca ca sera facile de rajouter les stats qu'il y avait avant
sub FilterMatrixR
{
my($in,$outdir,$out,$rh_ind2count,$ntags_min,$nind_min)=@_;

	my $rscript = "$outdir/ana.$nind_min.vs.$ntags_min.r";
	my $f_r = &GetStreamOut($rscript);
	select($f_r);

print<<END
print.stacks.sel<-function(file="stacks.sel",mat_res,ind_sel="NA",nind_min=1,ntags_min=1)
{
##mat_res       : presence-absence matrix (nloci X nsamples)
##lim_ntags     : minimum nb of loci for a sample to be included in the analysis
##ind_sel       : vector of selected samples
##nind_min      : minimum nb of samples required for a loci to be inclued in the analysis
  if(length(ind_sel)>1){mat_res=mat_res[,ind_sel]}
  mat_res[mat_res>0]=1
  num_tags=which(rowSums(mat_res[,colSums(mat_res)>ntags_min])>=nind_min)
  write.table(file="$out",num_tags,quote=F,col.names=F,row.names=F)
}

mat_res=as.matrix(read.table("$in",row.names=1,h=T))
print.stacks.sel(file="$in",mat_res=mat_res,nind_min=$nind_min,ntags_min=$ntags_min)
END
;
	$f_r->close;
	my $cde = "cd $outdir ; R --vanilla < $rscript > $rscript.stdout 2> $rscript.stderr ";
	&System($cde,${main::DEBUG},undef);
	select(STDOUT);
}

#============================================================================================================
#============================================================================================================
#============================================================================================================

sub TestParalogieR
{
my($outdir,$file,$seuil)=@_;

	my $currdir = $ENV{PWD};
	chdir($outdir);
	my($prefix)=$file=~/.+\/(.+)/;
	$prefix=~s/.matrix.tsv//;
	my $rscript = "$outdir/ana.npbloci-$seuil.r";
	my $f_r = &GetStreamOut($rscript);
	select($f_r);

print<<END
rm(list=ls()) # clear memory
file <- "$file"
mat <- as.matrix(read.table(file,row.names=1,h=T))
head(mat)
noms<-attr(mat,"dimnames")[[2]]
# JRAS03456.tags.tsv JRAS03779b.tags.tsv
# 1                  2                   0
# 2                  1                   1
# 3                  1                   7
dim(mat)
# [1] 3 2

#------------------------------------------------------
sur_mesure1<-function(x,seuil){return(which(x>=seuil))}
seuil<-$seuil
out <- apply(X=mat, MARGIN=1, FUN=sur_mesure1,seuil=seuil)
out
rm(mat) # clear
sur_mesure2<-function(l){
	vec <- l
	if(length(l)>0) {
        m<-max(unlist(l), na.rm = TRUE)
        vec2<-rep(NA,length=length(noms)) ; vec2[vec]<-1}
	
	if(length(l)==0) {vec2<-rep(NA,length=length(noms))}
	return(vec2)
}
out2 <- lapply(X=out, FUN=sur_mesure2)
effectif <- as.numeric(lapply(X=out,FUN=length))
rm(out)# clear
if(length(out2)>1) # rajout JG sinon ca plante s'il n'y en a aucun
{
resultats<-data.frame(matrix(unlist(out2),nrow=length(out2),byrow=TRUE))
names(resultats)<-noms
resultats\$effectif <- effectif
head(resultats)
summary(resultats\$effectif) -> summary_effectif
apply(resultats, MARGIN=2, FUN=sum, na.rm=TRUE) -> sum_each_col
file  <- paste("$prefix",seuil,".csv",sep="")
write.csv(x=as.matrix(resultats) ,file=file, quote=FALSE)
file2 <- paste("$prefix",seuil,"_summary_effectif.csv",sep="")
write(x=summary_effectif ,file=file2)
file3 <- paste("$prefix",seuil,"_sum_per_col.csv",sep="")
write.csv(x=sum_each_col ,file=file3, quote=FALSE)
rm(summary_effectif)
rm(sum_each_col)
}
END
;
	$f_r->close;
	my $cde = "cd $outdir ; R --vanilla < $rscript > $rscript.stdout 2> $rscript.stderr ";
	&System($cde,${main::DEBUG},undef);
	my @a_ls = `ls $outdir/$prefix*$seuil.csv 2> /dev/null`;
	chomp(@a_ls);
	my $ret = undef;
	if ( scalar(@a_ls) == 0 )
	{
		print STDERR "Warning: no potentially problematic loci found at cut-off=$seuil\n";
	}
	else
	{
		$ret = $a_ls[0];
	}
	select(STDOUT);
	chdir($currdir);
	return($ret);
}

#============================================================================================================
#============================================================================================================
#============================================================================================================

1;
